@@ -15,21 +15,18 @@
 
 """Classes representing various MJCF attribute data types."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import abc
 import collections
 import hashlib
+import io
 import os
 
 from dm_control.mjcf import base
 from dm_control.mjcf import constants
 from dm_control.mjcf import debugging
+from dm_control.mjcf import skin
 from dm_control.mujoco.wrapper import util
 import numpy as np
-import six
 
 # Copybara placeholder for internal file handling dependency.
 
@@ -37,7 +34,7 @@ from dm_control.utils import io as resources
 
 
 _INVALID_REFERENCE_TYPE = (
-    'Reference should be an MJCF Element whose type is one of {valid_types!r}: '
+    'Reference should be an MJCF Element whose type is {valid_type!r}: '
     'got {actual_type!r}.')
 
 _MESH_EXTENSIONS = ('.stl', '.msh')
@@ -48,8 +45,7 @@ _INVALID_MESH_EXTENSION = (
     .format(_MESH_EXTENSIONS))
 
 
-@six.add_metaclass(abc.ABCMeta)
-class _Attribute(object):
+class _Attribute(metaclass=abc.ABCMeta):
   """Abstract base class for MJCF attribute data types."""
 
   def __init__(self, name, required, parent, value,
@@ -168,8 +164,8 @@ class Keyword(_Attribute):
                conflict_allowed, conflict_behavior, valid_values):
     self._valid_values = collections.OrderedDict(
         (value.lower(), value) for value in valid_values)
-    super(Keyword, self).__init__(
-        name, required, parent, value, conflict_allowed, conflict_behavior)
+    super().__init__(name, required, parent, value, conflict_allowed,
+                     conflict_behavior)
 
   def _assign(self, value):
     if not value:
@@ -193,8 +189,8 @@ class Array(_Attribute):
                conflict_allowed, conflict_behavior, length, dtype):
     self._length = length
     self._dtype = dtype
-    super(Array, self).__init__(name, required, parent, value,
-                                conflict_allowed, conflict_behavior)
+    super().__init__(name, required, parent, value, conflict_allowed,
+                     conflict_behavior)
 
   def _assign(self, value):
     self._value = self._check_shape(np.array(value, dtype=self._dtype))
@@ -206,7 +202,7 @@ class Array(_Attribute):
     if self._value is None:
       return None
     else:
-      out = six.BytesIO()
+      out = io.BytesIO()
       # 17 decimal digits is sufficient to represent a double float without loss
       # of precision.
       # https://en.wikipedia.org/wiki/IEEE_754#Character_representation
@@ -275,8 +271,8 @@ class Reference(_Attribute):
   def __init__(self, name, required, parent, value,
                conflict_allowed, conflict_behavior, reference_namespace):
     self._reference_namespace = reference_namespace
-    super(Reference, self).__init__(
-        name, required, parent, value, conflict_allowed, conflict_behavior)
+    super().__init__(name, required, parent, value, conflict_allowed,
+                     conflict_behavior)
 
   def _check_dead_reference(self):
     if isinstance(self._value, base.Element) and self._value.is_removed:
@@ -285,7 +281,7 @@ class Reference(_Attribute):
   @property
   def value(self):
     self._check_dead_reference()
-    return super(Reference, self).value
+    return super().value
 
   @value.setter
   def value(self, new_value):
@@ -294,12 +290,13 @@ class Reference(_Attribute):
   @property
   def reference_namespace(self):
     if isinstance(self._reference_namespace, _Attribute):
-      return self._reference_namespace.value
+      return constants.INDIRECT_REFERENCE_ATTRIB.get(
+          self._reference_namespace.value, self._reference_namespace.value)
     else:
       return self._reference_namespace
 
   def _assign(self, value):
-    if not isinstance(value, (base.Element, six.string_types)):
+    if not isinstance(value, (base.Element, str)):
       raise ValueError(
           'Expect a string or `mjcf.Element` value: got {}'.format(value))
     elif not value:
@@ -308,16 +305,9 @@ class Reference(_Attribute):
       if isinstance(value, base.Element):
         value_namespace = (
             value.spec.namespace.split(constants.NAMESPACE_SEPARATOR)[0])
-        if isinstance(self._reference_namespace, _Attribute):
-          try:
-            self._reference_namespace.value = value_namespace
-          except ValueError:
-            raise ValueError(_INVALID_REFERENCE_TYPE.format(
-                valid_types=self._reference_namespace.valid_values,
-                actual_type=value_namespace))
-        elif value_namespace != self._reference_namespace:
+        if value_namespace != self.reference_namespace:
           raise ValueError(_INVALID_REFERENCE_TYPE.format(
-              valid_types=[self._reference_namespace],
+              valid_type=self.reference_namespace,
               actual_type=value_namespace))
       self._value = value
 
@@ -380,8 +370,8 @@ class BasePath(_Attribute):
   def __init__(self, name, required, parent, value,
                conflict_allowed, conflict_behavior, path_namespace):
     self._path_namespace = path_namespace
-    super(BasePath, self).__init__(
-        name, required, parent, value, conflict_allowed, conflict_behavior)
+    super().__init__(name, required, parent, value, conflict_allowed,
+                     conflict_behavior)
 
   def _assign(self, value):
     if not isinstance(value, str):
@@ -401,20 +391,12 @@ class BasePath(_Attribute):
     return None
 
 
-class Asset(object):
-  """Class representing a binary asset."""
+class BaseAsset:
+  """Base class for binary assets."""
 
-  __slots__ = ('contents', 'extension', 'prefix')
+  __slots__ = ('extension', 'prefix')
 
-  def __init__(self, contents, extension, prefix=''):
-    """Initializes a new `Asset`.
-
-    Args:
-      contents: The contents of the file as a bytestring.
-      extension: A string specifying the file extension (e.g. '.png', '.stl').
-      prefix: (optional) A prefix applied to the filename given in MuJoCo's VFS.
-    """
-    self.contents = contents
+  def __init__(self, extension, prefix=''):
     self.extension = extension
     self.prefix = prefix
 
@@ -441,21 +423,59 @@ class Asset(object):
     return filename + self.extension
 
 
+class Asset(BaseAsset):
+  """Class representing a binary asset."""
+
+  __slots__ = ('contents',)
+
+  def __init__(self, contents, extension, prefix=''):
+    """Initializes a new `Asset`.
+
+    Args:
+      contents: The contents of the file as a bytestring.
+      extension: A string specifying the file extension (e.g. '.png', '.stl').
+      prefix: (optional) A prefix applied to the filename given in MuJoCo's VFS.
+    """
+    self.contents = contents
+    super().__init__(extension, prefix)
+
+
+class SkinAsset(BaseAsset):
+  """Class representing a binary asset corresponding to a skin."""
+
+  __slots__ = ('skin', 'parent', '_cached_revision', '_cached_contents')
+
+  def __init__(self, contents, parent, extension, prefix=''):
+    self.skin = skin.parse(
+        contents, lambda body_name: parent.root.find('body', body_name))
+    self.parent = parent
+    self._cached_revision = -1
+    self._cached_contents = None
+    super().__init__(extension, prefix)
+
+  @property
+  def contents(self):
+    if self._cached_revision < self.parent.namescope.revision:
+      self._cached_contents = skin.serialize(self.skin)
+      self._cached_revision = self.parent.namescope.revision
+    return self._cached_contents
+
+
 class File(_Attribute):
   """Attribute representing an asset file."""
 
   def __init__(self, name, required, parent, value,
                conflict_allowed, conflict_behavior, path_namespace):
     self._path_namespace = path_namespace
-    super(File, self).__init__(name, required, parent, value,
-                               conflict_allowed, conflict_behavior)
+    super().__init__(name, required, parent, value, conflict_allowed,
+                     conflict_behavior)
     parent.namescope.files.add(self)
 
   def _assign(self, value):
     if not value:
       self.clear()
     else:
-      if isinstance(value, six.string_types):
+      if isinstance(value, str):
         asset = self._get_asset_from_path(value)
       elif isinstance(value, Asset):
         asset = value
@@ -486,9 +506,13 @@ class File(_Attribute):
       except KeyError:
         pass
       path_parts.append(path)
-      full_path = os.path.join(*path_parts)
+      full_path = os.path.join(*path_parts)  # pylint: disable=no-value-for-parameter
       contents = resources.GetResource(full_path)
-    return Asset(contents=contents, extension=extension, prefix=filename)
+    if self._parent.tag == constants.SKIN:
+      return SkinAsset(contents=contents, parent=self._parent,
+                       extension=extension, prefix=filename)
+    else:
+      return Asset(contents=contents, extension=extension, prefix=filename)
 
   def _validate_extension(self, extension):
     if self._parent.tag == constants.MESH:
